
"""
Movie recommender system as a showcase project
"""

import collections
import pandas as pd
import tensorflow as tf

def load_movielense():
    """Load an return movies and ratings as dataframes"""
    ratings_filename = "data/MovieLense20M/rating.csv"
    ratings_df = pd.read_csv(ratings_filename)

    movies_filename = "data/MovieLense20M/movie.csv"
    movies_df = pd.read_csv(movies_filename)

    return ratings_df, movies_df


def collect_user_histories(ratings, min_rating=2.1):
    """Create a per-user rating list"""
    user_histories = collections.defaultdict(lambda: [])  # dict mapping ids to movies

    ratings = ratings.sort_values(by=['userId', 'timestamp'])

    for user_id, movie_id, rating, _ in ratings.values:
        if rating >= min_rating:
            user_histories[user_id].append(movie_id)

    return user_histories


def create_user_examples(user_history, min_len=3, max_len=100):
    """Create examples by sliding a 3-100 window over the history"""
    examples = []
    for label_idx in range(min_len, len(user_history)):
        start_idx = max(0, label_idx - max_len)
        context_movie_ids = user_history[start_idx:label_idx]
        if len(context_movie_ids) < min_len:
            continue

        movie_id = user_history[label_idx]

        feature = {
            "context_movie_id": tf.train.Feature(
                int64_list=tf.train.Int64List(value=context_movie_ids)
             ),
             "label_movie_id": tf.train.Feature(
                int64_list=tf.train.Int64List(value=[movie_id])
             )
        }
        example = tf.train.Example(features=tf.train.Features(feature=feature))
        examples.append(example)

    return examples
